/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/rc4.h>

#ifndef WIN32
    #define strnicmp    strncasecmp
#endif

#ifndef u_char
    #define u_char  unsigned char
#endif



#define VER     "0.1"
#define BUFFSZ  1024
#define CONN    "[connection"
#define NAME    "connname="
#define HOST    "hostname="
#define PORT    "bfsmport="
#define USER    "bfsmuser="
#define PASS    "bfsmpass="



void std_err(void);
u_char *bfsmdec(u_char *data, int len);



int main(int argc, char *argv[]) {
    FILE    *fd;
    int     type;
    u_char  buff[BUFFSZ + 1];


    setbuf(stdout, NULL);

    fputs("\n"
        "BF1942 Server and Remote Manager passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\n"
            "Usage: %s <type> <file>\n"
            "\n"
            "Type:\n"
            "1 = for BFRemoteManager.ini\n"
            "2 = for Mods\\bf1942\\Settings\\UserAccess.con\n"
            "3 = for the data sniffed from the network (saved in a file).\n"
            "    The first 32 bytes sent from the client to the server are the encrypted\n"
            "    username while the next 32 bytes are the password (so your file must\n"
            "    contain one of the 2 fields or both).\n"
            "\n", argv[0]);
        exit(1);
    }

    type = atoi(argv[1]);
    if((type < 1) || (type > 3)) {
        fputs("\nError: you can choose only between 1 and 3\n\n", stdout);
        exit(1);
    }

    printf("- open file \"%s\"\n", argv[2]);
    fd = fopen(argv[2], "rb");
    if(!fd) std_err();

    if(type == 1) {
        fputs("- file format:   BFRemoteManager.ini\n\n", stdout);
        while(fgets(buff, BUFFSZ, fd)) {
            if(!strnicmp(buff, CONN, sizeof(CONN) - 1)) {
                fputc('\n', stdout);
            } else if(!strnicmp(buff, NAME, sizeof(NAME) - 1)) {
                printf("  Connection:   %s", buff + sizeof(NAME) - 1);
            } else if(!strnicmp(buff, HOST, sizeof(HOST) - 1)) {
                printf("  Hostname:     %s", buff + sizeof(HOST) - 1);
            } else if(!strnicmp(buff, PORT, sizeof(PORT) - 1)) {
                printf("  Port:         %s", buff + sizeof(PORT) - 1);
            } else if(!strnicmp(buff, USER, sizeof(USER) - 1)) {
                printf("  Username:     %s\n", bfsmdec(buff + sizeof(USER) - 1, 64));
            } else if(!strnicmp(buff, PASS, sizeof(PASS) - 1)) {
                printf("  Password:     %s\n", bfsmdec(buff + sizeof(PASS) - 1, 64));
            }
        }

    } else if(type == 2) {
        fputs("- file format:   Mods\\bf1942\\Settings\\UserAccess.con format\n\n", stdout);
        for(;;) {
            if(fread(buff, 64, 1, fd) != 1) break;
            printf("  Username:     %s\n", bfsmdec(buff, 64));
            if(fgetc(fd) != ',') break;

            if(fread(buff, 64, 1, fd) != 1) break;
            printf("  Password:     %s\n", bfsmdec(buff, 64));
            if(fgetc(fd) != ',') break;

            if(fread(buff, 66, 1, fd) != 1) break;
            printf("  CD-Key hash:  %s\n", bfsmdec(buff, 66));
            if(fgetc(fd) != ',') break;

            fgets(buff, BUFFSZ, fd);    // permissions
            printf("  Permissions:  %s\n", buff);
            // I'm too lazy however you can easily read the bits
            // to know the permissions of the user
        }

    } else {
        fputs("- file format:   raw sniffed data\n\n", stdout);
        if(fread(buff, 32, 1, fd) == 1) {
            printf("  Decoded:      %s\n", bfsmdec(buff, 32));
        }
        if(fread(buff, 32, 1, fd) == 1) {
            printf("  Decoded:      %s\n", bfsmdec(buff, 32));
        }
    }

    fclose(fd);
    fputs("\n- finished\n", stdout);
    return(0);
}



u_char *bfsmdec(u_char *data, int len) {
    const static u_char encdata[] =
        "\xA4\x0C\x48\xC7\x32\x3C\x36\x13\x11\x0E\xB7\x71\xBB\x7B\x82\x72"
        "\x68\x54\x4C\x72\x4D\xE8\x78\xB9\x6C\x63\x38\x77\x0D\xC7\x14\xC6"
        "\x86\x40\x75\x0B\x2E\x84\xC5\xC6\xB6\x6D\x18\x9F\x4C\x56\xFB\x0A"
        "\x99\xDA\x7E\xC1\x79\x65\x9D\x53\xB3\x87\xBA\x17\xE6\x5A\x04\x03"
        "\xEB\x98\x32\x60\xC8\x17\x63\x21\x07\x0C\x95\x84\x1A\x03\xAA\xB2"
        "\xE2\x8D\x81\xEF\x2C\x77\xAE\xBC\xB8\xAE\xF1\xC9\x16\xD0\xB8\x71"
        "\x7A\x6D\xFB\x2A\xB3\xDD\x5D\x29\x9C\x1A\x53\xBE\x44\x19\xB1\xA9"
        "\x69\x46\xB8\x33\x6C\x8B\xEB\xDB\xAF\x0A\x0D\xA9\x03\xB3\x41\x5E"
        "\x0A\x56\x6F\xAC\x59\xCC\xB9\x12\x4F\x98\xC4\x21\x73\x57\x39\x99"
        "\x69\x5D\x3E\x49\xA4\xEA\x38\xE6\x00\xCE\x65\xC2\x0F\x51\x99\xF8"
        "\xA6\xAD\x53\xB8\xD0\x44\x18\xE7\xE7\x9D\x7F\xC1\xFC\x1F\x89\x6F"
        "\x69\x96\x4E\xA0\x7A\xAC\x7D\x3B\x9E\x7D\x1F\x05\xF0\xBF\x10\x24"
        "\x95\xB4\x3D\xB9\xEA\x08\x70\xAD\xE5\x75\xAC\xDC\x87\xA3\x9F\xC4"
        "\x62\x24\x96\x2E\xA7\xD1\x05\x44\xD2\x82\xD2\x6C\x9F\x9D\x7C\x96"
        "\x2A\x83\x6D\xEE\xB2\x3D\x67\x34\xDB\x75\xD4\x3B\xB5\xE6\x70\x7A"
        "\x7C\x4A\xE3\xA0\xEF\xF8\x1B\x08\x50\xCA\xE0\xAB\x57\x4E\xF8\x67"
        "\x11\xCE\x48\xD9\x75\x9E\x89\x46\x2B\x98\x66\x93\xE6\xFD\x7D\x9A"
        "\x36\x51\xEA\xD0\x40\xBE\x70\xE0\xF1\x19\x47\x61\x5D\x9A\x35\xFA"
        "\x72\xA7\xD2\xB7\xB9\x7A\xDC\x20\xB3\xC5\xA2\x7B\x40\xFC\xB2\xB2"
        "\xDF\xE1\xCB\x5A\x3A\x4C\xF1\x9C\xB7\x20\x45\xD5\x00\x00\x99\x44"
        "\xBA\x76\x0D\xDD\x91\x10\x25\xF5\x9C\x42\x9B\xE3\xFE\x0D\x72\x11"
        "\x66\x5B\x9E\xDE\xD5\xDA\x8A\xED\xC1\xC6\x87\x06\xD9\xC9\x2B\xB7"
        "\x4A\x93\x96\x32\x1C\x05\x89\x82\xB8\x3A\xAF\x9F\xB3\x78\x12\x31"
        "\x5C\x8A\x45\xBB\xC1\xF9\x6A\x35\xCC\x40\x23\xBE\xA2\x52\x35\xFA"
        "\x3D\x5B\x86\xFB\x16\xEB\xAD\x3E\x8B\x72\x5B\xCB\x55\x39\xE0\xFC"
        "\x4E\xDB\x32\xD6\x0C\xDC\x03\x78\x2B\x5B\x18\xE3\xEF\x53\x5E\xFC"
        "\xAD\xB9\xEA\x25\xE7\x51\xC0\x09\xD6\x1E\x12\x20\x65\xBD\xF4\xF4"
        "\xBE\xF5\x50\x75\x3D\xEA\x54\xDB\x34\xE9\xDA\xC6\x59\xC9\xCF\x47"
        "\x32\xCD\xFD\xD2\x1F\xB3\xA0\x90\xAD\xF4\x9B\x67\x7E\xDD\xB6\x16"
        "\x74\x84\x43\x29\x18\xC2\xDE\x45\x4B\x62\x1C\x35\x80\x1B\x29\xB5"
        "\xC1\x43\xC4\x5F\x5C\x06\x17\x08\xE3\xE4\x87\xA7\xF3\x85\x08\x48"
        "\x0E\xE7\xDA\xA6\x73\xDC\x20\xFA\x22\xF1\x83\xF1\xA2\x87\x67\xD0"
        "\x91\x69\xEC\x49\x6D\x9E\x34\x99\x58\xD4\x91\x77\xEA\xE9\x51\xBB"
        "\xC2\x64\x80\x2F\x29\x2C\xD7\xC1\xF3\x6E\xBB\xCF\x36\x45\x30\x3A"
        "\x67\x08\x7F\x70\x27\xBE\x90\xDA\x83\x7B\x47\xF3\xFD\x1A\xC4\x44"
        "\xFA\xC5\xE7\x6F\x7F\x02\x34\x36\x9C\xE3\x7A\x66\x36\x0B\x64\xC2"
        "\x6D\x53\x89\x1B\x04\x97\x18\xE6\x84\x76\x55\xCF\xDF\x66\x19\xE3"
        "\xB8\xC1\xFD\x90\x67\x57\x35\xA4\x6D\x08\x5A\xB5\x53\x4F\x99\xE4"
        "\x00\x63\xBD\x22\xBF\x5B\x3F\x1D\x18\xEB\x3C\x99\x72\xD0\x8E\x48"
        "\x6F\x7D\x88\xA0\x68\xF6\xC4\x65\xE2\xC0\x79\x24\x45\x24\x76\xEA"
        "\x25\x0A\x91\x0F\x10\x1D\xDC\xBE\xE8\xD8\x05\xFD\xED\x7A\x90\x7D"
        "\xC6\x47\x92\xA0\xC6\xA0\x1B\x2C\xD5\xAD\x02\x49\xA5\x08\xA7\x66"
        "\x96\x96\xC5\x17\xD2\x5D\xD2\x48\xD5\x53\xAB\x92\x13\x1D\xA4\x26"
        "\xB9\xC6\x7E\xE4\x26\x42\x0B\x53\x05\xCC\x73\xD1\xCC\x8B\x75\x88"
        "\x67\xB6\xF7\x59\x07\xF4\x8A\x8F\xE0\xA5\x58\xD6\xBC\xEB\x44\xB0"
        "\x22\x0D\xB2\xB8\xDC\xA4\x9F\xBE\x1F\xA4\xE4\xEB\xF7\x3F\xDA\xE2"
        "\x46\x23\x9C\x3C\x50\x37\x0B\xCA\xCC\x44\xB2\x4A\x69\xB4\x16\xBF"
        "\xF0\x96\x29\x60\x4F\xEC\xAF\xB2\x16\xA9\xED\x49\xC7\x13\x35\x56"
        "\xE7\xE6\xAF\x02\x15\x94\x0F\xDE\xD8\xC5\xA2\x82\xFD\x84\x2D\x6F"
        "\xF1\x49\xC3\xF6\x2F\x4C\x66\x06\x89\xBD\x35\x55\xCF\x00\xE5\x1D"
        "\xC0\x3E\x53\xA6\xF9\x1C\x6E\xFE\xDA\xB8\x6F\xD2\xA9\x5F\x78\x76"
        "\x89\x9D\x22\xC1\x06\x2E\xB1\xCB\x24\x3C\x12\x0E\xAB\x89\x2F\xEF"
        "\x80\x76\xC8\xED\x96\x9B\xA7\xA4\x7A\x8B\x20\xBF\x0C\x82\x3F\xE3"
        "\x39\x32\xBB\x58\x39\x24\x78\x2A\x24\x85\x87\xAD\xE5\xE8\xB7\x72"
        "\x76\x72\x7B\x02\x74\x01\x3F\x7A\x98\xEF\xFA\x0B\xCC\x5D\x88\x36"
        "\x84\xB7\xD1\x23\x26\x24\x16\x18\xD3\x16\x93\x5E\xE3\xEF\xB5\x37"
        "\x0A\x27\x35\xFA\xDD\x40\xBB\x8D\xDD\xA5\x4C\x86\xBB\x4F\x76\x83"
        "\x2D\xC9\xC9\xAC\xE8\xC0\x97\x46\x29\x60\x88\x6C\x10\x15\x8F\x34"
        "\x21\xA4\x29\x7E\x7D\x0A\xD7\x38\x56\x90\x08\x71\x88\x6E\x11\x8B"
        "\x65\x3C\x4D\xE0\x94\x5C\xC1\x86\x3D\xE0\xA2\xF6\xF3\x48\x5C\xD9"
        "\xF9\x4E\x0B\x47\x17\x78\x6F\xB5\xB1\x14\x9B\x85\x96\x94\xF8\x26"
        "\xF2\x80\xB2\xE3\x2D\xFE\x0C\x2B\x41\xDB\x5D\xA7\x41\xEA\x8D\xB1"
        "\x08\x0C\xA2\x7B\xE2\x5D\x8D\x15\xF7\x84\x5F\xC1\xEB\x54\x89\x62"
        "\x56\x0D\xCF\xE5\x4C\x92\xCA\x09\x73\xC5\xFE\x7B\xC8\x0D\xA8\x61";
    int     i,
            tmp;
    u_char  *out;
    RC4_KEY rc4key;

    if(len > 32) {
        out = data;
        len >>= 1;
        for(i = 0; i < len; i++) {
            sscanf(data, "%02X", &tmp);
            out[i] = tmp;
            data += 2;
        }
        out[i] = 0;
        data = out;
    }

    RC4_set_key(&rc4key, sizeof(encdata) - 1, encdata);
    RC4(&rc4key, len, data, data);
    return(data);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}

